/**
 * Svelte action to activate validation framework for the form.
 * @module data/form-controller
 */
import type { RuleDefinition } from "../data/common";
import type { JQueryApi } from "../data/dom-jquery";
import type { FormSettings } from "../data/semantic-types";
export type FormApi = {
    form(settings?: FormSettings): void;
    form(command: string, arg1?: unknown, arg2?: unknown): unknown;
};
/** Controls Semantic UI form element and it's data validation.
    Hides implementation details of SuiFormController from the FieldController. */
export interface FormController {
    addRule: (key: string, rules: RuleDefinition) => void;
    removeRule: (key: string, rules: RuleDefinition) => void;
    doValidateField: (key: string) => void;
    doValidateForm: () => void;
    markForValidation(key: string): void;
    validateIfMarked(): boolean;
}
/**
 * Form validation controller. Is accessed by both `<FormValidation>` and individual `<InitField>`.
 * */
export declare class SuiFormController implements FormController {
    formId: string;
    elem: JQueryApi & FormApi;
    isActive: boolean;
    rules: Record<string, RuleDefinition>;
    hasToValidate: boolean;
    constructor(elem: JQueryApi);
    private activateRule;
    private deactivateRule;
    addRule(key: string, rule: RuleDefinition): void;
    removeRule(key: string, rule: RuleDefinition): void;
    setActive(newValue: boolean): void;
    doValidateField(key: string): void;
    doValidateForm(): void;
    markForValidation(key: string): void;
    validateIfMarked(): boolean;
}
/** Adds validation rule to the field.
    Common class used by all Init* components to control field validation */
export declare class FieldController {
    key: string | undefined;
    formCtrl?: FormController;
    rules?: RuleDefinition;
    constructor(elem: JQueryApi, validationRules?: RuleDefinition);
    /** Validate the new field value, if the field is validated (has form controller).
        This method is `async` as it debounces (deduplicates) the validation event
        in case multiple fields are modified in a single Svelte "tick". */
    revalidate(): Promise<void>;
    /** Remove rules and revalidate */
    removeRules(): void;
}
/** Imperatively call form validation, may be passed by name in event handler from within the form.
    In this case it will find the parent form using event target.
    Or may be called with form id attribute as a string. */
export declare function validateForm(e: MouseEvent | KeyboardEvent | string): void;
